/* Inspector.h - Copyright 1993 Steve Ludtke */
/* This object controls the inspector panel. Most of the actual */
/* manipulations are performed by MolObj */

#import <appkit/appkit.h>

@interface Inspector:Object
{
    id	abcount;	/* atoms/bonds count display */
    id	aminoIns;	/* amino acid inspector box */
    id	atabIns;	/* atomic table inspector box */
    id	atomIns;	/* atom inspector box */
    id	inspectBox;	/* inspector display box */

/* a_... methods and variables are for the atom inspector */
    id	a_bond;		/* list of bonded atoms */
    id	a_message;	/* message display space */
    id	a_pos1;		/* x,y,z position of atom */
    id	a_pos2;		/* x,y position of atom (unused) */
    id	a_select;	/* atom SelectView */

/* t_... methods and variables are for the atomic table inspector */
    id	t_color;	/* color display */
    id	t_message;	/* message space */
    id	t_radii;	/* radius display */
    id	t_select;	/* SelectView */

/* m_... methods and variables are for the amino acid inspector */
	id m_select;	/* SelectView */
	id m_angS;		/* angle sliders */
	id m_angT;		/* angle text displays */
	id m_amview;	/* points to AminoView (protein tool) */

	id molobj;		/* points to MolObj */
	Molecule *mol;	/* the molecule data structure */
	struct ELINFO *elinfo;	/* atomic table */
	int nmol;		/* # molecules */
	char mode;		/* inspector mode */
	struct SELDAT *list;	/* one SELDAT structure is used for all SelectViews */
	int nlist,nsel;	/* # entries in list and # selected */
	float size;		/* current atom display scaling factor */
}

- startup:(Molecule *)Mol :(int)Nmol :(struct ELINFO *)Elinfo;
- update:(int)Nmol :(int)level;	/* update message from MolObj */
- newMol;

- a_insp:sender;		/* initialize atom inspector */
- a_bondAng:sender;		/* calculate & display bond angle */
- a_bondLen:sender;		/* calculate & display bond length */
- a_dihedral:sender;	/* calculate & display dihedral */

- t_insp:sender;		/* initialize atomic table inspector */
- t_newColor:sender;	/* atom color changed */
- t_newRad:sender;		/* atom radius changed */
- t_revert:sender;		/* revert to data in covalent.rad */
- t_save:sender;		/* save to user's covalent.rad */
- t_newSize:sender;		/* change the global display radius scaling */
- t_update:sender;		/* make changes visible */

- m_insp:sender;		/* initialize amino acid inspector */
- m_upd:sender;			/* reflect new dihedrals in SelectView */
- m_setAngS:sender;		/* new angle via slider */
- m_setAngT:sender;		/* new angle via text object */
- m_xfer:sender;		/* send amino acid list to Protein Tool */

- selected:(int)n :(int)nsel;	/* called by SelectView */
- deselected:(int)n :(int)nsel;

- newInsp;	/* clean up current inspector (prep for new one) */
@end
